local name, addon = ...

local embed = addon['LibInit']("zzCommon-Frames", 2, {'AddDrag'})
if(type(embed) ~= 'function') then return end

local function saveFrame(self)
	local db = self['db']
	local fn = self:GetName()
	db[fn] = db[fn] or {
		['xpos'] = 0,
		['ypos'] = 0,
	}
	local x, y = self:GetCenter() 
	local s = self:GetEffectiveScale()
	local uis = UIParent:GetScale()
	x = x*s - GetScreenWidth()*uis/2
	y = y*s - GetScreenHeight()*uis/2
	db[fn]['xpos'] = addon['round'](x/uis,0)
	db[fn]['ypos'] = addon['round'](y/uis,0)
end

local function loadFrame() end -- dummy

local schedules = {}
local scheduleframe = CreateFrame("Frame")
scheduleframe:SetScript("OnEvent", function(self, event)
	for i=1,#schedules do
		schedules[i]['func'](schedules[i]['frame'])
	end
	wipe(schedules)
	self:UnregisterEvent()
end)

local function addSchedule(self, func)
	schedules[#schedules + 1] = {
		['frame'] = self,
		['func'] = func,
	}
	if(#schedules == 1) then
		scheduleframe:RegisterEvent("PLAYER_REGEN_ENABLED")
	end
end

loadFrame = function(self) -- local because of previous dummy
	if not self then return end
	if( InCombatLockdown() ) then
		addSchedule(self, loadFrame)
		return
	end
	self:SetMovable(true)
	local db = self['db']
	local fn = self:GetName()
	db[fn] = db[fn] or {
		['xpos'] = 0,
		['ypos'] = 0,
	}
	local s = self:GetEffectiveScale()
	local uis = UIParent:GetScale()
	local x = db[fn]['xpos']
	local y = db[fn]['ypos']
	self:ClearAllPoints()
	self:SetPoint("CENTER", UIParent, "CENTER", x*uis/s, y*uis/s)
	self:SetUserPlaced(true)
end

local function toggle(self)
	if(self:IsVisible()) then
		self:Hide()
	else
		self:Show()
	end
end

local function center(self)
	self:ClearAllPoints()
	self:SetPoint("Center", UIParent,"Center",0,0)
	saveFrame(self)
end

local function startMove(self)
	if InCombatLockdown() then return end
	if(self['locked']) then return end
	if(self['moveparent']) then
		self:GetParent():StartMoving()
	else
		self:StartMoving()
	end
end

local function stopMove(self)
	if(self['moveparent']) then
		self:GetParent():StopMovingOrSizing()
		saveFrame(self:GetParent())
	else
		self:StopMovingOrSizing()
		saveFrame(self)
	end
end

local function dragView(self)
	if(IsAltKeyDown() and self:GetParent()['dragAble']()) then
		self['tex']:SetTexture(0, .5, 0, .5)
		self:Show()
	else
		if(self:IsVisible()) then
			self['tex']:SetTexture(0, 1, 0 , 0)
			stopMove(self)
			self:Hide()
		end
	end
end

local lib = {
	['AddDrag'] = function(self,typ,tab)
		tab = tab or {
			['width'] = 32,
			['height'] = 32,
			['anchor'] = "BOTTOMLEFT",
			['parentanchor'] = "TOPLEFT",
			['xOffset'] = 0,
			['yOffset'] = 0,
			['offset'] = 16,
		}
		loadFrame(self)
		self['dragframe'] = self['dragframe'] or CreateFrame("Frame","$PARENTdrag",self)
		self['dragframe']:SetFrameStrata("LOW")
		self:SetMovable(true)
		self['dragframe']:SetMovable(true)
		self['dragframe']:EnableMouse(true)
		self['dragframe'].moveparent = true
		self['dragframe']:SetScript("OnMouseDown", startMove)
		self['dragframe']:SetScript("OnMouseUp", stopMove)
		if(typ == "BOX") then
			self['dragframe']:SetWidth(tab['width'])
			self['dragframe']:SetHeight(tab['height'])
			self['dragframe']:SetPoint(tab['anchor'], self, tab['parentanchor'], tab['xOffset'], tab['yOffset'])
		else
			local offset = tab['offset']
			self['dragframe']:SetWidth(self:GetWidth() + offset)
			self['dragframe']:SetHeight(self:GetHeight() + offset)
			self['dragframe']:SetPoint("TOPLEFT", self, "TOPLEFT", -offset/2, offset/2)
		end
		self['dragframe']['tex'] = self['dragframe']['tex'] or self['dragframe']:CreateTexture()
		self['dragframe']['tex']:SetAllPoints(self.dragframe)
		self['dragframe']['tex']:SetTexture(0,0,0,0)
		self['dragframe']:RegisterEvent("MODIFIER_STATE_CHANGED")
		self['dragframe']:SetScript("OnEvent",dragView)
		self['dragframe']:Hide()
	end,
}

embed(lib)